#include "cctk.h"
#include "cctk_Parameters.h"
#include "cctk_Arguments.h"
#include "cctk_Functions.h"
#include <math.h>
#include <stdio.h>

#include "util_Table.h"
#include <assert.h>

extern "C" { 
  void CoreCollapseControl_LocalSMax(CCTK_ARGUMENTS);
  void CoreCollapseControl_GlobalSMax(CCTK_ARGUMENTS);
}


void CoreCollapseControl_LocalSMax(CCTK_ARGUMENTS) 
{
  DECLARE_CCTK_ARGUMENTS;
  DECLARE_CCTK_PARAMETERS;

  int nx = cctk_lsh[0];
  int ny = cctk_lsh[1];
  int nz = cctk_lsh[2];
  int i,j,k;


  *local_entropy_max = 0.0e0;

  if(!(*in_prebounce) && !(*force_check)) return;

  if(!*evolve_temper) {
    CCTK_WARN(0,"You must not set corecollapsecontrol::bounce_criterion = entropy if not actually using a hot EOS");
  }

  for (k=0;k<nz;k++)
    for (j=0;j<ny;j++)
      for (i=0;i<nx;i++) {

	int index = CCTK_GFINDEX3D(cctkGH,i,j,k);

	if(r[index] <= 100.0e0) {
	  if(entropy[index] > *local_entropy_max) {
	    *local_entropy_max = entropy[index];
	  }
	}
      }

  return;
}

void CoreCollapseControl_GlobalSMax(CCTK_ARGUMENTS) 
{
  DECLARE_CCTK_ARGUMENTS;
  DECLARE_CCTK_PARAMETERS;

  int vartype;
  int varindex = -1;
  int ierr = 0;
  int reduction_handle;

  if( ((((cctk_iteration) % rho_max_every) != 0) && !(*force_check) )|| (*bounce)) 
    return;

  if(!(*in_prebounce) && !(*force_check)) return;

// lets find the maximum entropy on the grid
  varindex = CCTK_VarIndex("CoreCollapseControl::local_entropy_max");
  vartype = CCTK_VarTypeI(varindex);
  reduction_handle = CCTK_ReductionHandle("maximum");
  ierr = CCTK_Reduce(cctkGH,-1,reduction_handle,1,vartype,global_entropy_max,1,varindex);
  assert(!ierr);

  return;
}
